import pickle
from zoneinfo import ZoneInfo

from parameterized import parameterized  # type: ignore[import-untyped]

from heliclockter import datetime_local, datetime_tz, datetime_utc
from tests.shared import datetime_cet


@parameterized.expand(
    [
        datetime_utc(2021, 1, 1, 10, 0, 0, tzinfo=ZoneInfo("UTC")),
        datetime_cet(2021, 1, 1, 10, 0, 0, tzinfo=ZoneInfo("CET")),
        datetime_tz(2021, 1, 1, 10, 0, 0, tzinfo=ZoneInfo("UTC")),
        datetime_tz(2021, 1, 1, 10, 0, 0, tzinfo=ZoneInfo("EST")),
        datetime_tz(2021, 1, 1, 10, 0, 0, tzinfo=ZoneInfo("CET")),
        datetime_local.now(),
    ]
)
def test_pickling(dt: datetime_tz) -> None:
    """Test that datetime_tz objects can be pickled and unpickled correctly."""
    # Pickle the datetime
    pickled = pickle.dumps(dt)

    # Unpickle the datetime
    unpickled = pickle.loads(pickled)

    # Check that the unpickled object is of the same class
    assert isinstance(unpickled, type(dt))

    # Check that all attributes are preserved
    assert unpickled.year == dt.year
    assert unpickled.month == dt.month
    assert unpickled.day == dt.day
    assert unpickled.hour == dt.hour
    assert unpickled.minute == dt.minute
    assert unpickled.second == dt.second
    assert unpickled.microsecond == dt.microsecond
    assert unpickled.tzinfo == dt.tzinfo
    assert unpickled.fold == dt.fold


def test_pickling_with_fold() -> None:
    """Test pickling with fold attribute set."""
    # Create a datetime with fold=1
    dt = datetime_cet(2021, 10, 31, 2, 30, 0, tzinfo=ZoneInfo("CET"), fold=1)

    # Pickle and unpickle
    pickled = pickle.dumps(dt)
    unpickled = pickle.loads(pickled)

    # Check that fold is preserved
    assert dt.fold == 1
    assert unpickled.fold == 1
    assert isinstance(unpickled, datetime_cet)


def test_pickling_complex_scenarios() -> None:
    """Test pickling with more complex scenarios."""
    # Test with microseconds
    dt_micro = datetime_utc(2021, 1, 1, 10, 0, 0, 123456, tzinfo=ZoneInfo("UTC"))
    pickled = pickle.dumps(dt_micro)
    unpickled = pickle.loads(pickled)
    assert unpickled.microsecond == 123456

    # Test with different timezones
    dt_est = datetime_tz(2021, 1, 1, 10, 0, 0, tzinfo=ZoneInfo("EST"))
    pickled = pickle.dumps(dt_est)
    unpickled = pickle.loads(pickled)
    assert unpickled.tzinfo == ZoneInfo("EST")


def test_pickling_edge_cases() -> None:
    """Test pickling with edge cases."""
    # Test with very early date
    early_dt = datetime_utc(1900, 1, 1, 0, 0, 0, tzinfo=ZoneInfo("UTC"))
    pickled = pickle.dumps(early_dt)
    unpickled = pickle.loads(pickled)
    assert unpickled == early_dt

    # Test with very late date
    late_dt = datetime_utc(2100, 12, 31, 23, 59, 59, 999999, tzinfo=ZoneInfo("UTC"))
    pickled = pickle.dumps(late_dt)
    unpickled = pickle.loads(pickled)
    assert unpickled == late_dt

    # Test with leap year date
    leap_dt = datetime_utc(2020, 2, 29, 12, 0, 0, tzinfo=ZoneInfo("UTC"))
    pickled = pickle.dumps(leap_dt)
    unpickled = pickle.loads(pickled)
    assert unpickled == leap_dt


def test_pickling_different_protocols() -> None:
    """Test pickling with different pickle protocols."""
    dt = datetime_utc(2021, 1, 1, 10, 0, 0, tzinfo=ZoneInfo("UTC"))

    # Test with protocol 4 (Python 3.4+)
    pickled_4 = pickle.dumps(dt, protocol=4)
    unpickled_4 = pickle.loads(pickled_4)
    assert unpickled_4 == dt

    # Test with protocol 5 (Python 3.8+)
    pickled_5 = pickle.dumps(dt, protocol=5)
    unpickled_5 = pickle.loads(pickled_5)
    assert unpickled_5 == dt


def test_pickling_with_complex_timezones() -> None:
    """Test pickling with complex timezone objects."""
    # Test with timezone that has DST rules
    dt_cet = datetime_cet(2021, 7, 1, 12, 0, 0, tzinfo=ZoneInfo("CET"))
    pickled = pickle.dumps(dt_cet)
    unpickled = pickle.loads(pickled)
    assert unpickled.tzinfo == ZoneInfo("CET")

    # Test with timezone that has offset
    dt_est = datetime_tz(2021, 1, 1, 10, 0, 0, tzinfo=ZoneInfo("EST"))
    pickled = pickle.dumps(dt_est)
    unpickled = pickle.loads(pickled)
    assert unpickled.tzinfo == ZoneInfo("EST")


def test_pickling_roundtrip_equality() -> None:
    """Test that pickling and unpickling preserves exact equality."""
    test_cases = [
        datetime_utc(2021, 1, 1, 10, 0, 0, tzinfo=ZoneInfo("UTC")),
        datetime_cet(2021, 6, 15, 14, 30, 45, 123456, tzinfo=ZoneInfo("CET")),
        datetime_tz(2021, 12, 31, 23, 59, 59, 999999, tzinfo=ZoneInfo("EST")),
        datetime_local.now(),
    ]

    for dt in test_cases:
        pickled = pickle.dumps(dt)
        unpickled = pickle.loads(pickled)

        # Check exact equality
        assert unpickled == dt

        # Check that all attributes match
        assert unpickled.year == dt.year
        assert unpickled.month == dt.month
        assert unpickled.day == dt.day
        assert unpickled.hour == dt.hour
        assert unpickled.minute == dt.minute
        assert unpickled.second == dt.second
        assert unpickled.microsecond == dt.microsecond
        assert unpickled.tzinfo == dt.tzinfo
        assert unpickled.fold == dt.fold


def test_pickling_subclass_preservation() -> None:
    """Test that pickling preserves the exact subclass type."""
    # Test datetime_utc
    utc_dt = datetime_utc(2021, 1, 1, 10, 0, 0, tzinfo=ZoneInfo("UTC"))
    pickled = pickle.dumps(utc_dt)
    unpickled = pickle.loads(pickled)
    assert isinstance(unpickled, datetime_utc)

    # Test datetime_cet
    cet_dt = datetime_cet(2021, 1, 1, 10, 0, 0, tzinfo=ZoneInfo("CET"))
    pickled = pickle.dumps(cet_dt)
    unpickled = pickle.loads(pickled)
    assert isinstance(unpickled, datetime_cet)

    local_dt = datetime_local.now()
    pickled = pickle.dumps(local_dt)
    unpickled = pickle.loads(pickled)
    assert isinstance(unpickled, datetime_local)



# pylint: disable=import-outside-toplevel
from __future__ import annotations

import datetime as _datetime
from typing import TYPE_CHECKING, Any, ClassVar, TypeVar, cast
from zoneinfo import ZoneInfo

# `date` and `timedelta` are exposed for your convenience in case this module is used in combination
# with an import linter that prohibits importing the `datetime` package anywhere.
date = _datetime.date
timedelta = _datetime.timedelta


tz_local = cast("ZoneInfo", _datetime.datetime.now().astimezone().tzinfo)

__version__ = "3.0.0"


DateTimeTzT = TypeVar("DateTimeTzT", bound="datetime_tz")
IntFloat = int | float


class DatetimeTzError(ValueError):
    """
    An error with the input value when trying to create or mutate a `datetime_tz` instance.
    """


class datetime_tz(_datetime.datetime):
    """
    A `datetime_tz` is just a `datetime.datetime` but which is guaranteed to be timezone aware.
    """

    assumed_timezone_for_timezone_naive_input: ClassVar[ZoneInfo | None] = None

    if TYPE_CHECKING:

        def __init__(
            self,
            year: int,
            month: int,
            day: int,
            hour: int = 0,
            minute: int = 0,
            second: int = 0,
            microsecond: int = 0,
            *,
            tzinfo: _datetime.tzinfo,
            fold: int = 0,
        ) -> None:
            pass

    else:

        def __init__(  # pylint: disable=unused-argument
            self,
            year: int,
            month: int = None,
            day: int = None,
            hour: int = 0,
            minute: int = 0,
            second: int = 0,
            microsecond: int = 0,
            tzinfo: _datetime.tzinfo = None,
            fold: int = 0,
        ) -> None:
            msg = f"{self.__class__} must have a timezone"
            assert self.tzinfo is not None, msg
            tz_expected = self.assumed_timezone_for_timezone_naive_input or self.tzinfo

            msg = f"{self.__class__} got invalid timezone {self.tzinfo!r}, expected {tz_expected!r}"
            assert self.tzinfo == tz_expected, msg

            self.assert_aware_datetime(self)

    # We don't require pydantic as a dependency, but add validate logic if it exists.
    try:
        import pydantic

        if pydantic.__version__[0] != "2":
            raise RuntimeError("Unexpected Pydantic version, expected 2.x")

        if TYPE_CHECKING:
            from pydantic import GetCoreSchemaHandler, GetJsonSchemaHandler
            from pydantic.json_schema import JsonSchemaValue
            from pydantic_core import CoreSchema

        @classmethod
        def __get_pydantic_core_schema__(cls, _: Any, __: GetCoreSchemaHandler) -> CoreSchema:
            from pydantic_core import core_schema

            from_datetime_schema = core_schema.chain_schema(
                [
                    core_schema.datetime_schema(),
                    core_schema.no_info_plain_validator_function(cls.from_datetime),
                ]
            )

            return core_schema.json_or_python_schema(
                json_schema=from_datetime_schema,
                python_schema=core_schema.union_schema(
                    [
                        core_schema.is_instance_schema(datetime_tz),
                        from_datetime_schema,
                    ]
                ),
            )

        @classmethod
        def __get_pydantic_json_schema__(
            cls, _core_schema: CoreSchema, handler: GetJsonSchemaHandler
        ) -> JsonSchemaValue:
            from pydantic_core import core_schema

            return handler(core_schema.datetime_schema())

    except ImportError:
        pass

    @classmethod
    def from_datetime(cls: type[DateTimeTzT], dt: _datetime.datetime) -> DateTimeTzT:
        # Case datetime is naive and there is no assumed timezone.
        if dt.tzinfo is None and cls.assumed_timezone_for_timezone_naive_input is None:
            raise DatetimeTzError("Cannot create aware datetime from naive if no tz is assumed")

        # Case: datetime is naive, but the timezone is assumed.
        if dt.tzinfo is None:
            dt = dt.replace(tzinfo=cls.assumed_timezone_for_timezone_naive_input)

        # Case: datetime is aware and the timezone is assumed, enforce that timezone.
        elif (assumed_tz := cls.assumed_timezone_for_timezone_naive_input) is not None:
            # Case: when `assumed_timezone_for_timezone_naive_input` is declared on the input
            # dt it cannot be instantiated in a different timezone.
            if getattr(dt, "assumed_timezone_for_timezone_naive_input", None) is not None:
                dt = _datetime.datetime(
                    year=dt.year,
                    month=dt.month,
                    day=dt.day,
                    hour=dt.hour,
                    minute=dt.minute,
                    second=dt.second,
                    microsecond=dt.microsecond,
                    tzinfo=dt.tzinfo,
                    fold=dt.fold,
                ).astimezone(tz=assumed_tz)

            else:
                dt = dt.astimezone(assumed_tz)

        cls.assert_aware_datetime(dt)
        return cls(
            year=dt.year,
            month=dt.month,
            day=dt.day,
            hour=dt.hour,
            minute=dt.minute,
            second=dt.second,
            microsecond=dt.microsecond,
            tzinfo=dt.tzinfo,  # type: ignore[arg-type]
            fold=dt.fold,
        )

    @classmethod
    def now(cls: type[DateTimeTzT], tz: _datetime.tzinfo | None = None) -> DateTimeTzT:
        tz = cls.assumed_timezone_for_timezone_naive_input or tz
        if tz is None:
            raise DatetimeTzError(
                "Must override assumed_timezone_for_timezone_naive_input "
                "or give a timezone when calling now"
            )
        return cls.from_datetime(_datetime.datetime.now(tz))

    @classmethod
    def future(
        cls: type[DateTimeTzT],
        weeks: IntFloat = 0,
        days: IntFloat = 0,
        hours: IntFloat = 0,
        minutes: IntFloat = 0,
        seconds: IntFloat = 0,
        milliseconds: IntFloat = 0,
        microseconds: IntFloat = 0,
        tz: ZoneInfo | None = None,
    ) -> DateTimeTzT:
        delta = timedelta(
            weeks=weeks,
            days=days,
            hours=hours,
            minutes=minutes,
            seconds=seconds,
            milliseconds=milliseconds,
            microseconds=microseconds,
        )
        return cls.now(tz=tz) + delta

    @classmethod
    def past(
        cls: type[DateTimeTzT],
        weeks: IntFloat = 0,
        days: IntFloat = 0,
        hours: IntFloat = 0,
        minutes: IntFloat = 0,
        seconds: IntFloat = 0,
        milliseconds: IntFloat = 0,
        microseconds: IntFloat = 0,
        tz: ZoneInfo | None = None,
    ) -> DateTimeTzT:
        delta = timedelta(
            weeks=weeks,
            days=days,
            hours=hours,
            minutes=minutes,
            seconds=seconds,
            milliseconds=milliseconds,
            microseconds=microseconds,
        )
        return cls.now(tz=tz) - delta

    @classmethod
    def fromisoformat(cls: type[DateTimeTzT], date_string: str) -> DateTimeTzT:
        return cls.from_datetime(_datetime.datetime.fromisoformat(date_string))

    @classmethod
    def strptime(cls: type[DateTimeTzT], date_string: str, __format: str) -> DateTimeTzT:
        dt = _datetime.datetime.strptime(date_string, __format)
        return cls.from_datetime(dt)

    @staticmethod
    def assert_aware_datetime(dt: _datetime.datetime) -> None:
        """
        Check that the given `datetime.datetime` instance is timezone aware. Throws
        an AssertionError otherwise.
        """
        assert dt.tzinfo is not None and dt.tzinfo.utcoffset(dt) is not None

    def astimezone(self, tz: _datetime.tzinfo | None = None) -> datetime_tz:
        """
        Return a datetime_tz object with the same datetime data but in the specified timezone.
        Uses local timezone if no timezone is provided.
        """
        if tz is None:
            tz = tz_local
        if tz is self.tzinfo:
            return self
        return datetime_tz.fromtimestamp(self.timestamp(), tz=tz)

    def __deepcopy__(self: DateTimeTzT, memodict: object) -> DateTimeTzT:
        """
        Deepcopy does not natively work with the __init__ we add to this class
        for extra assertions. Therefore, we override it.
        """
        return self.__class__(
            year=self.year,
            month=self.month,
            day=self.day,
            hour=self.hour,
            minute=self.minute,
            second=self.second,
            microsecond=self.microsecond,
            tzinfo=self.tzinfo,  # type: ignore[arg-type]
            fold=self.fold,
        )


class datetime_utc(datetime_tz):
    """
    A `datetime_utc` is a `datetime_tz` but which is guaranteed to be in the UTC+0 timezone.
    """

    assumed_timezone_for_timezone_naive_input = ZoneInfo("UTC")

    @classmethod
    def fromtimestamp(cls, timestamp: float) -> datetime_utc:  # type: ignore[override]  # pylint: disable=arguments-differ
        """
        Parses a timestamp to a timezone aware datetime.
        """
        return cls.from_datetime(_datetime.datetime.fromtimestamp(timestamp, tz=ZoneInfo("UTC")))


class datetime_local(datetime_tz):
    """
    A `datetime_local` is a `datetime_tz` but which is guaranteed to be in the local timezone.
    """

    assumed_timezone_for_timezone_naive_input = tz_local



from zoneinfo import ZoneInfo

from parameterized import parameterized  # type: ignore[import-untyped]

from heliclockter import datetime_tz, datetime_utc


@parameterized.expand(
    [
        (
            datetime_utc(2021, 1, 10, 9, 00, 00, tzinfo=ZoneInfo("UTC")),
            "%Y-%m-%d",
            "2021-01-10",
        ),
        (
            datetime_utc(2021, 1, 10, 9, 00, 00, tzinfo=ZoneInfo("UTC")),
            "%Y-%m-%d %H:%M",
            "2021-01-10 09:00",
        ),
        (
            datetime_utc(2021, 1, 10, 9, 00, 00, tzinfo=ZoneInfo("UTC")),
            "%Y-%m-%d %H:%M:%S",
            "2021-01-10 09:00:00",
        ),
        (
            datetime_utc(2021, 1, 10, 9, 00, 00, tzinfo=ZoneInfo("UTC")),
            "%Y-%m-%d %H:%M:%SZ",
            "2021-01-10 09:00:00Z",
        ),
        (
            datetime_utc(2021, 1, 10, 9, 00, 00, tzinfo=ZoneInfo("UTC")),
            "%Y-%m-%dT%H:%M:%S.%f",
            "2021-01-10T09:00:00.000000",
        ),
    ]
)
def test_strftime(dt: datetime_tz, fmt: str, expected_str: str) -> None:
    formatted_str = dt.strftime(fmt)
    assert formatted_str == expected_str



from zoneinfo import ZoneInfo

from pydantic import BaseModel

from heliclockter import datetime_tz


class DatetimeTZModel(BaseModel):
    dt: datetime_tz


def test_pydantic_serialization() -> None:
    obj = DatetimeTZModel(dt=datetime_tz(2021, 1, 10, 10, 0, tzinfo=ZoneInfo("UTC")))

    assert obj.model_dump_json() == '{"dt":"2021-01-10T10:00:00Z"}'



from zoneinfo import ZoneInfo

from parameterized import parameterized  # type: ignore[import-untyped]

from heliclockter import datetime_local, datetime_tz, datetime_utc, tz_local
from tests.shared import datetime_cet


@parameterized.expand(
    [
        # Test astimezone on datetime_utc
        (
            datetime_utc(2021, 1, 1, 10, 0, 0, tzinfo=ZoneInfo("UTC")),
            ZoneInfo("CET"),
            datetime_tz(2021, 1, 1, 11, 0, 0, tzinfo=ZoneInfo("CET")),
        ),
        (
            datetime_utc(2021, 1, 1, 10, 0, 0, tzinfo=ZoneInfo("UTC")),
            ZoneInfo("EST"),
            datetime_tz(2021, 1, 1, 5, 0, 0, tzinfo=ZoneInfo("EST")),
        ),
        (
            datetime_utc(2021, 1, 1, 10, 0, 0, tzinfo=ZoneInfo("UTC")),
            ZoneInfo("UTC"),
            datetime_tz(2021, 1, 1, 10, 0, 0, tzinfo=ZoneInfo("UTC")),
        ),
        # Test astimezone on datetime_cet
        (
            datetime_cet(2021, 1, 1, 10, 0, 0, tzinfo=ZoneInfo("CET")),
            ZoneInfo("UTC"),
            datetime_tz(2021, 1, 1, 9, 0, 0, tzinfo=ZoneInfo("UTC")),
        ),
        (
            datetime_cet(2021, 1, 1, 10, 0, 0, tzinfo=ZoneInfo("CET")),
            ZoneInfo("EST"),
            datetime_tz(2021, 1, 1, 4, 0, 0, tzinfo=ZoneInfo("EST")),
        ),
        (
            datetime_cet(2021, 1, 1, 10, 0, 0, tzinfo=ZoneInfo("CET")),
            ZoneInfo("CET"),
            datetime_tz(2021, 1, 1, 10, 0, 0, tzinfo=ZoneInfo("CET")),
        ),
        # Test astimezone on datetime_tz
        (
            datetime_tz(2021, 1, 1, 10, 0, 0, tzinfo=ZoneInfo("UTC")),
            ZoneInfo("CET"),
            datetime_tz(2021, 1, 1, 11, 0, 0, tzinfo=ZoneInfo("CET")),
        ),
        (
            datetime_tz(2021, 1, 1, 10, 0, 0, tzinfo=ZoneInfo("CET")),
            ZoneInfo("UTC"),
            datetime_tz(2021, 1, 1, 9, 0, 0, tzinfo=ZoneInfo("UTC")),
        ),
    ]
)
def test_astimezone(
    original_dt: datetime_tz, target_tz: ZoneInfo, expected_dt: datetime_tz
) -> None:
    """Test that astimezone returns the correct datetime with proper timezone conversion."""
    result = original_dt.astimezone(target_tz)

    # Check that the result is of the same class as the original
    assert isinstance(result, datetime_tz)

    # Check that the timezone conversion is correct
    assert result.year == expected_dt.year
    assert result.month == expected_dt.month
    assert result.day == expected_dt.day
    assert result.hour == expected_dt.hour
    assert result.minute == expected_dt.minute
    assert result.second == expected_dt.second
    assert result.microsecond == expected_dt.microsecond
    assert result.tzinfo == expected_dt.tzinfo


def test_astimezone_converts_type() -> None:
    """Test that astimezone converts to datetime_tz when changing timezones."""
    utc_dt = datetime_utc(2021, 1, 1, 10, 0, 0, tzinfo=ZoneInfo("UTC"))
    result_utc = utc_dt.astimezone(ZoneInfo("CET"))
    assert isinstance(result_utc, datetime_tz)

    cet_dt = datetime_cet(2021, 1, 1, 10, 0, 0, tzinfo=ZoneInfo("CET"))
    result_cet = cet_dt.astimezone(ZoneInfo("UTC"))
    assert isinstance(result_cet, datetime_tz)

    local_dt = datetime_local.now()
    result_local = local_dt.astimezone(ZoneInfo("CET"))
    assert isinstance(result_local, datetime_tz)

    tz_dt = datetime_tz(2021, 1, 1, 10, 0, 0, tzinfo=ZoneInfo("UTC"))
    result_tz = tz_dt.astimezone(ZoneInfo("CET"))
    assert isinstance(result_tz, datetime_tz)


def test_astimezone_same_timezone() -> None:
    """Test astimezone when converting to the same timezone."""
    dt = datetime_utc(2021, 1, 1, 10, 0, 0, tzinfo=ZoneInfo("UTC"))

    # Convert to the same timezone
    result = dt.astimezone(ZoneInfo("UTC"))

    assert isinstance(result, datetime_tz)
    assert result == dt  # Should be identical
    assert result.tzinfo == ZoneInfo("UTC")


def test_astimezone_with_different_datetime_types() -> None:
    """Test astimezone behavior with different datetime_tz subclasses."""
    # Test datetime_utc
    utc_dt = datetime_utc(2021, 1, 1, 10, 0, 0, tzinfo=ZoneInfo("UTC"))
    utc_to_cet = utc_dt.astimezone(ZoneInfo("CET"))
    assert type(utc_to_cet) is datetime_tz  # pylint: disable=unidiomatic-typecheck
    assert utc_to_cet.tzinfo == ZoneInfo("CET")
    assert utc_to_cet.hour == utc_dt.hour + 1

    # Test datetime_cet
    cet_dt = datetime_cet(2021, 1, 1, 10, 0, 0, tzinfo=ZoneInfo("CET"))
    cet_to_utc = cet_dt.astimezone(ZoneInfo("UTC"))
    assert type(cet_to_utc) is datetime_tz  # pylint: disable=unidiomatic-typecheck
    assert cet_to_utc.tzinfo == ZoneInfo("UTC")
    assert cet_to_utc.hour == cet_dt.hour - 1

    # Test datetime_local
    local_dt = datetime_local.now()
    local_to_cet = local_dt.astimezone(ZoneInfo("CET"))
    assert type(local_to_cet) is datetime_tz  # pylint: disable=unidiomatic-typecheck
    assert local_to_cet.tzinfo == ZoneInfo("CET")


def test_astimezone_timezone_conversion_accuracy() -> None:
    """Test that astimezone correctly handles timezone conversions."""
    # Test UTC to various timezones
    utc_dt = datetime_utc(2021, 1, 1, 12, 0, 0, tzinfo=ZoneInfo("UTC"))

    # UTC to EST (UTC-5)
    est_result = utc_dt.astimezone(ZoneInfo("EST"))
    assert est_result.hour == 7  # 12:00 UTC - 5 hours = 7:00 EST

    # UTC to CET (UTC+1 in January)
    cet_result = utc_dt.astimezone(ZoneInfo("CET"))
    assert cet_result.hour == 13  # 12:00 UTC + 1 hour = 13:00 CET

    # UTC to JST (UTC+9)
    jst_result = utc_dt.astimezone(ZoneInfo("Asia/Tokyo"))
    assert jst_result.hour == 21  # 12:00 UTC + 9 hours = 21:00 JST


@parameterized.expand(
    [
        datetime_utc(2021, 1, 1, 10, 0, 0, tzinfo=ZoneInfo("UTC")),
        datetime_cet(2021, 1, 1, 10, 0, 0, tzinfo=ZoneInfo("CET")),
        datetime_tz(2021, 1, 1, 10, 0, 0, tzinfo=ZoneInfo("UTC")),
        datetime_local.now(),
    ]
)
def test_astimezone_without_arguments(dt: datetime_tz) -> None:
    result = dt.astimezone()
    assert isinstance(result, datetime_tz)
    assert result.tzinfo == tz_local

    result2 = dt.astimezone(None)
    assert isinstance(result2, datetime_tz)
    assert result2.tzinfo == tz_local

    assert dt.timestamp() == result.timestamp() == result2.timestamp()
